@@ -15,16 +15,39 @@ __correspondence_email__ = 'alexander.juestel@rwth-aachen.de'
 
 __affiliations__ = 'CGRE - RWTH Aachen University'
 
-__version_date__ = '2021-01-03'
+__version_date__ = '2021-01-30'
 
-__version__ = '0.1.5'
+__version__ = '0.1.6'
 
-__changelog__ = """What is new in version 0.1.5:
+__changelog__ = """What is new in version 0.1.6:
+- Added Tutorial 39 and functions to work with Shapely Objects containing Z components
+- Added Tutorial 40 and functions to work with GPX data
+- Added Tutorial 41 and functions to work with KML data
+- Added Tutorial 42 and functions to drape linestrings over PyVista meshes
+- Added Tutorial 43 and functions to create linestrings from PyVista contours
+- Added Tutorial 44 and functions to fit a plane through earthquake hypocenters
+- Added Tutorial 45 and functions to open ESRI Grids and Petrel ZMAP Grids
+- Added Tutorial 46 on how to work th HGT rasters 
+- Added Tutorial 47 on how to perform Delaunay triangulation with Shapely
+- Added Tutorial 48 on how to georeference a raster using rasterio
+- Added Tutorial 49 on how to slice GemPy model lith blocks with PyVista
+- Added Tutorial 50 on how to work with well data from Leapfrog
+- Added Tutorial 51 on how to assign properties to the GemPy lith block
+- Added Tutorial 52 on how to pick data from PyVista meshes
+- Starting refactoring of functions to support PyGeos
+- Added Binder support to repo
+- Extended extract_xyz function to work for a GDF consisting of Points, LineStrings and Polygons with Z components
+- Added notes and comments to docstring examples
+- Added path checks to code
+- Added utility functions
+- Bug fixes
+"""
+
+__changelogs__ = {'0.1.5': """What is new in version 0.1.5:
 - Major refactoring of the API 
 - Adding a readthedocs Documentation page
-- Adding more tutorials"""
-
-__changelogs__ = {'0.1.4': """What is new in version 0.1.4:
+- Adding more tutorials""",
+                  '0.1.4': """What is new in version 0.1.4:
 Major refactoring of the API for vector.py and raster.py
 Adding a readthedocs Documentation page""",
                   '0.1.3': """What is new in version 0.1.3: 
