@@ -19,6 +19,7 @@ GNU General Public License (LICENSE.md) for more details.
 
 """
 
+import os
 import json
 import geopandas as gpd
 import numpy as np
@@ -34,6 +35,7 @@ from gemgis import vector
 from sklearn.neighbors import NearestNeighbors
 import geopy
 import pyproj
+import pygeos
 
 import gempy as gp
 
@@ -66,6 +68,7 @@ def to_section_dict(gdf: gpd.geodataframe.GeoDataFrame,
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> import geopandas as gpd
         >>> gdf = gpd.read_file(filename='file.shp')
@@ -74,6 +77,7 @@ def to_section_dict(gdf: gpd.geodataframe.GeoDataFrame,
         0	None	POINT (695.467 3.226)	    Section1
         1	None	POINT (669.284 1060.822)	Section1
 
+        >>> # Creating Section dict
         >>> section_dict = gg.utils.to_section_dict(gdf=gdf, section_column='Section')
         >>> section_dict
         {'Section1': ([695.4667461080886, 3.2262250771374283],
@@ -152,6 +156,7 @@ def convert_to_gempy_df(gdf: gpd.geodataframe.GeoDataFrame,
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> import geopandas as gpd
         >>> import rasterio
@@ -164,12 +169,15 @@ def convert_to_gempy_df(gdf: gpd.geodataframe.GeoDataFrame,
         3   None    Ton         POINT (157.812 615.999)
         4   None    Ton         POINT (191.318 719.094)
 
+        >>> # Loading Digital Elevation Model
         >>> dem = rasterio.open(fp='dem.tif')
         >>> dem
         <open DatasetReader name='dem.tif' mode='r'>
 
+        >>> # Defining extent
         >>> extent = [0, 972, 0, 1069]
 
+        >>> # Converting GeoDataFrame to DataFrame
         >>> df = gg.utils.convert_to_gempy_df(gdf=gdf, dem=dem, extent=extent)
         >>> df
             formation	X	Y	Z
@@ -292,6 +300,7 @@ def set_extent(minx: Union[int, float] = 0,
     Example
     _______
 
+        >>> # Loading Libraries and setting the extent
         >>> import gemgis as gg
         >>> extent = gg.utils.set_extent(minx=0, maxx=972, miny=0, maxy=1069, minz=0, maxz=1000)
         >>> extent
@@ -315,7 +324,7 @@ def set_extent(minx: Union[int, float] = 0,
             extent = [minx, maxx, miny, maxy, minz, maxz]
 
     # Create extent from gdf of geom_type polygon
-    elif all(gdf.geom_type == "Polygon"):
+    elif all(pygeos.get_type_id(pygeos.from_shapely(gdf.geometry)) == 2):
 
         # Checking if the gdf is of type GeoDataFrame
         bounds = gdf.bounds.round().values.tolist()[0]
@@ -356,6 +365,7 @@ def set_resolution(x: int,
     Example
     _______
 
+        >>> # Loading Libraries and setting the resolution
         >>> import gemgis as gg
         >>> res = gg.utils.set_resolution(x=50, y=50, z=50)
         >>> res
@@ -419,6 +429,7 @@ def read_csv_as_gdf(path: str,
     Example
     _______
 
+        >>> # Loading Libraries and File as GeoDataFrame
         >>> import gemgis as gg
         >>> gdf = gg.utils.read_csv_as_gdf(path='file.csv')
         >>> gdf
@@ -435,6 +446,17 @@ def read_csv_as_gdf(path: str,
     if not isinstance(path, str):
         raise TypeError('Path must be provided as string')
 
+    # Getting the absolute path
+    path = os.path.abspath(path=path)
+
+    # Checking that the file has the correct file ending
+    if not path.endswith(".csv"):
+        raise TypeError("The raster must be saved as .csv file")
+
+    # Checking that the file exists
+    if not os.path.exists(path):
+        raise FileNotFoundError('File not found')
+
     # Checking that the x column is of type string
     if not isinstance(x, str):
         raise TypeError('X column name must be provided as string')
@@ -494,6 +516,7 @@ def show_number_of_data_points(geo_model: gp.core.model.Project):
     Example
     _______
 
+        >>> # Loading Libraries and displaying surface DataFrame of a GemPy geo_model
         >>> import gemgis as gg
         >>> geo_model.surfaces
             surface     series	        order_surfaces  color   id
@@ -501,6 +524,7 @@ def show_number_of_data_points(geo_model: gp.core.model.Project):
         1   Ton         Strat_Series	2               #9f0052 2
         2   basement    Strat_Series	3               #ffbe00 3
 
+        >>> # Adding number of data points to DataFrame
         >>> gg.utils.show_number_of_data_points(geo_model=geo_model)
             surface     series          order_surfaces  color   id  No. of Interfaces   No. of Orientations
         0   Sand1       Strat_Series    1               #015482 1   95                  0
@@ -633,11 +657,13 @@ def parse_categorized_qml(qml_name: str) -> tuple:
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> column, classes = gg.utils.parse_categorized_qml(qml_name='style.qml')
         >>> column
         'formation'
 
+        >>> # Inspecting classes
         >>> classes
         {'Sand1': {'border_width_map_unit_scale': '3x:0,0,0,0,0,0',
         'color': '179,90,42,255',
@@ -664,6 +690,17 @@ def parse_categorized_qml(qml_name: str) -> tuple:
     if not isinstance(qml_name, str):
         raise TypeError('Path must be of type string')
 
+    # Getting the absolute path
+    path = os.path.abspath(path=qml_name)
+
+    # Checking that the file has the correct file ending
+    if not path.endswith(".qml"):
+        raise TypeError("The raster must be saved as .qml file")
+
+    # Checking that the file exists
+    if not os.path.exists(path):
+        raise FileNotFoundError('File not found')
+
     # Opening the file
     with open(qml_name, "rb") as f:
         qml = xmltodict.parse(f)
@@ -706,11 +743,13 @@ def build_style_dict(classes: dict) -> dict:
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> column, classes = gg.utils.parse_categorized_qml(qml_name='style.qml')
         >>>column
         'formation'
 
+        >>> # Inspecting classes
         >>> classes
         {'Sand1': {'border_width_map_unit_scale': '3x:0,0,0,0,0,0',
         'color': '179,90,42,255',
@@ -724,6 +763,7 @@ def build_style_dict(classes: dict) -> dict:
         'outline_width_unit': 'MM',
         'style': 'solid'},....}
 
+        >>> # Creating Style Dict
         >>> style_dict = gg.utils.build_style_dict(classes=classes)
         >>> style_dict
         {'Sand1': {'color': '#b35a2a',
@@ -788,9 +828,12 @@ def load_surface_colors(path: str,
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> import geopandas as gpd
         >>> gdf = gpd.read_file(filename='file.shp')
+
+        >>> # Loading surface colors
         >>> colors = gg.utils.load_surface_colors(path='style.qml', gdf=gdf)
         >>> colors
         ['#b35a2a', '#b35a2a', '#525252']
@@ -808,6 +851,17 @@ def load_surface_colors(path: str,
     if not isinstance(path, str):
         raise TypeError('path must be provided as string')
 
+    # Getting the absolute path
+    path = os.path.abspath(path=path)
+
+    # Checking that the file has the correct file ending
+    if not path.endswith(".qml"):
+        raise TypeError("The raster must be saved as .qml file")
+
+    # Checking that the file exists
+    if not os.path.exists(path):
+        raise FileNotFoundError('File not found')
+
     # Checking that the gdf is of type GeoDataFrame
     if not isinstance(gdf, gpd.geodataframe.GeoDataFrame):
         raise TypeError('object must be of type GeoDataFrame')
@@ -854,6 +908,7 @@ def create_surface_color_dict(path: str) -> dict:
     Example
     _______
 
+        >>> # Loading Libraries and File
         >>> import gemgis as gg
         >>> surface_colors_dict = gg.utils.create_surface_color_dict(path='style.qml')
         >>> surface_colors_dict
@@ -872,6 +927,17 @@ def create_surface_color_dict(path: str) -> dict:
     if not isinstance(path, str):
         raise TypeError('path must be provided as string')
 
+    # Getting the absolute path
+    path = os.path.abspath(path=path)
+
+    # Checking that the file has the correct file ending
+    if not path.endswith(".qml"):
+        raise TypeError("The raster must be saved as .qml file")
+
+    # Checking that the file exists
+    if not os.path.exists(path):
+        raise FileNotFoundError('File not found')
+
     # Parse qml
     columns, classes = parse_categorized_qml(qml_name=path)
 
@@ -906,6 +972,7 @@ def get_location_coordinate(name: str) -> geopy.location.Location:
     Example
     _______
 
+        >>> # Loading Libraries and get location object
         >>> import gemgis as gg
         >>> location = gg.utils.get_location_coordinate(name='Aachen')
         >>> location
@@ -955,11 +1022,13 @@ def transform_location_coordinate(coordinates: geopy.location.Location,
     Example
     _______
 
+        >>> # Loading Libraries and get location object
         >>> import gemgis as gg
         >>> location = gg.utils.get_location_coordinate(name='Aachen')
         >>> location
         Location(Aachen, Städteregion Aachen, Nordrhein-Westfalen, Deutschland, (50.776351, 6.083862, 0.0))
 
+        >>> # Transforming location coordinates
         >>> result_dict = gg.utils.transform_location_coordinate(coordinates=location, crs='EPSG:4647')
         >>> result_dict
         {'Aachen, Städteregion Aachen, Nordrhein-Westfalen, Deutschland': (32294411.33488576, 5629009.357074926)}
@@ -1011,11 +1080,13 @@ def create_polygon_from_location(coordinates: geopy.location.Location) -> shapel
     Example
     _______
 
+        >>> # Loading Libraries and get location object
         >>> import gemgis as gg
         >>> location = gg.utils.get_location_coordinate(name='Aachen')
         >>> location
         Location(Aachen, Städteregion Aachen, Nordrhein-Westfalen, Deutschland, (50.776351, 6.083862, 0.0))
 
+        >>> # Creating polygon from location bounds
         >>> polygon = gg.utils.create_polygon_from_location(coordinates=location)
         >>> polygon.wkt
         'POLYGON ((50.8572449 5.9748624, 50.8572449 6.2180747, 50.6621373 6.2180747, 50.6621373 5.9748624, 50.8572449 5.9748624))'
@@ -1063,6 +1134,7 @@ def get_locations(names: Union[list, str],
     Example
     _______
 
+        >>> # Loading Libraries and get location objects
         >>> import gemgis as gg
         >>> names = ['Aachen', 'Cologne', 'Munich', 'Berlin']
         >>> location_dict = gg.utils.get_locations(names=names, crs='EPSG:4647')
@@ -1116,6 +1188,142 @@ def get_locations(names: Union[list, str],
     return result_dict
 
 
+def convert_location_dict_to_gdf(location_dict: dict) -> gpd.geodataframe.GeoDataFrame:
+    """Converting a location dict to a GeoDataFrame
+
+    Parameters
+    __________
+
+        location_dict : dict
+            Dict containing the name of the location and the coordinates
+
+    Returns
+    _______
+
+        gdf : gpd.geodataframe.GeoDataFrame
+            GeoDataFrame containing the location name and the coordinates of the location
+
+
+    Example
+    _______
+
+        >>> # Loading Libraries
+        >>> import gemgis as gg
+
+        >>> # Creating a dict with coordinates
+        >>> coordinates_dict = gg.utils.get_locations(names = ['Aachen', 'Berlin', 'München', 'Hamburg', 'Köln'], crs='EPSG:4647')
+
+        >>> # Converting dict to GeoDataFrame
+        >>> gdf = gg.utils.convert_location_dict_to_gdf(location_dict=coordinates_dict)
+        >>> gdf
+            City    X           Y           geometry
+        0	Aachen  32294411.33 5629009.36  POINT (32294411.335 5629009.357)
+        1	Berlin  32797738.56 5827603.74  POINT (32797738.561 5827603.740)
+        2	München 32691595.36 5334747.27  POINT (32691595.356 5334747.274)
+        3	Hamburg 32566296.25 5933959.96  POINT (32566296.251 5933959.965)
+        4	Köln    32356668.82 5644952.10  POINT (32356668.818 5644952.100)
+
+    """
+
+    # Checking that the input data is of type dict
+    if not isinstance(location_dict, dict):
+        raise TypeError('Input data must be provided as dict')
+
+    # Creating GeoDataFrame
+    gdf = gpd.GeoDataFrame(data=location_dict).T.reset_index()
+
+    # Assigning column names
+    gdf.columns = ['City', 'X', 'Y']
+
+    # Split city names to only show the name of the city
+    gdf['City'] = [i.split(',')[0] for i in gdf['City'].to_list()]
+
+    # Recreate GeoDataFrame and set coordinates as geometry objects
+    gdf = gpd.GeoDataFrame(data=gdf,
+                           geometry=gpd.points_from_xy(x=gdf['X'],
+                                                            y=gdf['Y']))
+
+    return gdf
+
+
+# Misc
+############################
+
+
+def assign_properties(lith_block: np.ndarray,
+                      property_dict: dict) -> np.ndarray:
+    """Replacing lith block IDs with physical properties
+
+    Parameters
+    __________
+
+        lith_block : np.ndarray
+            GemPy lith block array containing the surface IDs
+
+        property_array : dict
+            Dict containing the property values mapped to a surface ID
+
+    Returns
+    _______
+
+        property_block : np.ndarray
+            Array containing the properties
+
+    Example
+    _______
+
+        >>> # Loading Libraries and lith_block plus reshaping
+        >>> import gemgis as gg
+        >>> import numpy as np
+        >>> lith_block = np.load('lith_block.npy').reshape(50,50,50)
+
+        >>> # Defining properties
+        >>> density_values = [0.1, 2.5, 3.0]
+
+        >>> # Creating dict
+        >>> density_dict = {k: v for k,v in zip(np.unique(np.round(lith_block)), density_values)}
+        >>> density_dict
+        {1.0: 0.1, 2.0: 2.5, 3.0: 3.0}
+
+        >>> # Assign properties
+        >>> property_block = gg.utils.assign_properties(lith_block=lith_block, property_dict=property_dict)
+
+    """
+
+    # Checking that the lith block is a NumPy array
+    if not isinstance(lith_block, np.ndarray):
+        raise TypeError('Lith block must be a NumPy Array')
+
+    # Checking that the properties dict is a dict
+    if not isinstance(property_dict, dict):
+        raise TypeError('Properties must be provided as dict')
+
+    # Store shape
+    shape = lith_block.shape
+
+    # Creating arrays from key and values
+    k = np.array(list(property_dict.keys()))
+    v = np.array(list(property_dict.values()))
+
+    # Sorting the keys
+    sidx = k.argsort()
+
+    # Apply sorting to keys and values
+    k = k[sidx]
+    v = v[sidx]
+
+    # Create property block
+    idx = np.searchsorted(k, lith_block.ravel()).reshape(lith_block.shape)
+    idx[idx == len(k)] = 0
+    mask = k[idx] == lith_block
+    property_block = np.where(mask, v[idx], 0)
+
+    # Reshaping block
+    property_block = property_block.reshape(shape)
+
+    return property_block
+
+
 # TODO Refactor
 def get_nearest_neighbor(x: np.ndarray, y: np.ndarray) -> np.int64:
     """Function to return the index of the nearest neighbor for a given point y
