@@ -63,6 +63,7 @@ def load_wms(url: str) -> owslib.wms.WebMapService:
     Example
     _______
 
+        >>> # Loading Libraries and WMS Service
         >>> import gemgis as gg
         >>> wms = gg.web.load_wms(url='https://ows.terrestris.de/osm/service?')
         >>> wms
@@ -100,7 +101,9 @@ def load_as_map(url: str,
                 filetype: str,
                 transparent: bool = True,
                 save_image: bool = False,
-                path: str = None) -> owslib.util.ResponseWrapper:
+                path: str = None,
+                overwrite_file: bool = False,
+                create_directory: bool = False) -> owslib.util.ResponseWrapper:
     """Loading a portion of a WMS as array
 
     Parameters
@@ -138,6 +141,15 @@ def load_as_map(url: str,
         path : str
             Path and file name of the file to be saved, e.g. ``path=map.tif``
 
+        overwrite_file : bool
+            Variable to overwrite an already existing file.
+            Options include: ``True`` or ``False``, default set to ``False``
+
+        create_directory : bool
+            Variable to create a new directory of directory does not exist
+            Options include: ``True`` or ``False``, default set to ``False``
+
+
     Returns
     _______
 
@@ -147,6 +159,7 @@ def load_as_map(url: str,
     Example
     _______
 
+        >>> # Loading Libraries and WMS Service as Map
         >>> import gemgis as gg
         >>> wms_map = gg.web.load_as_map(url='https://ows.terrestris.de/osm/service?', layer='OSM-WMS', style='default', crs='EPSG:4647', bbox=[32286000,32328000, 5620000,5648000], size=[4200, 2800], filetype='image/png')
         >>> wms_map
@@ -182,7 +195,7 @@ def load_as_map(url: str,
 
     # Checking the length of the bbox list
     if len(bbox) != 4:
-        raise ValueError('Provide xmin, xmax, ymin, and ymax values for the bounding box')
+        raise ValueError('Provide minx, maxx, miny, and maxy values for the bounding box')
 
     # Checking if size is of type list
     if not isinstance(size, list):
@@ -208,6 +221,30 @@ def load_as_map(url: str,
     if not isinstance(path, (str, type(None))):
         raise TypeError('Path must be of type string')
 
+    if isinstance(path, str):
+        # Getting the absolute path
+        path = os.path.abspath(path=path)
+
+        # Checking that the file has the correct file ending
+        if not path.endswith(".png"):
+            if not path.endswith(".tif"):
+                raise TypeError("The raster must be saved as .png or .tif file")
+
+        # Getting path to directory
+        path_dir = os.path.dirname(path)
+
+        # Creating new directory
+        if not os.path.exists(path_dir):
+            if create_directory:
+                os.makedirs(path_dir)
+            else:
+                raise LookupError('Directory not found. Pass create_directory=True to create a new directory')
+
+        if not overwrite_file:
+            if os.path.exists(path):
+                raise FileExistsError(
+                    "The file already exists. Pass overwrite_file=True to overwrite the existing file")
+
     # Loading WMS Service
     wms = load_wms(url)
 
@@ -240,7 +277,9 @@ def load_as_array(url: str,
                   filetype: str,
                   transparent: bool = True,
                   save_image: bool = False,
-                  path: str = None) -> np.ndarray:
+                  path: str = None,
+                  overwrite_file: bool = False,
+                  create_directory: bool = False) -> np.ndarray:
     """Loading a portion of a WMS as array
 
     Parameters
@@ -278,6 +317,14 @@ def load_as_array(url: str,
         path : str
             Path and file name of the file to be saved, e.g. ``path=map.tif``
 
+        overwrite_file : bool
+            Variable to overwrite an already existing file.
+            Options include: ``True`` or ``False``, default set to ``False``
+
+        create_directory : bool
+            Variable to create a new directory of directory does not exist
+            Options include: ``True`` or ``False``, default set to ``False``
+
     Returns
     _______
 
@@ -287,6 +334,7 @@ def load_as_array(url: str,
     Example
     _______
 
+        >>> # Loading Libraries and WMS Service as array
         >>> import gemgis as gg
         >>> wms_map = gg.web.load_as_array(url='https://ows.terrestris.de/osm/service?', layer='OSM-WMS', style='default', crs='EPSG:4647', bbox=[32286000,32328000, 5620000,5648000], size=[4200, 2800], filetype='image/png')
         >>> wms_map
@@ -328,7 +376,7 @@ def load_as_array(url: str,
 
     # Checking the length of the bbox list
     if len(bbox) != 4:
-        raise ValueError('Provide xmin, xmax, ymin, and ymax values for the bounding box')
+        raise ValueError('Provide minx, maxx, miny and maxy values for the bounding box')
 
     # Checking if size is of type list
     if not isinstance(size, list):
@@ -354,6 +402,30 @@ def load_as_array(url: str,
     if not isinstance(path, (str, type(None))):
         raise TypeError('Path must be of type string')
 
+    if isinstance(path, str):
+        # Getting the absolute path
+        path = os.path.abspath(path=path)
+
+        # Checking that the file has the correct file ending
+        if not path.endswith(".png"):
+            if not path.endswith(".tif"):
+                raise TypeError("The raster must be saved as .png or .tif file")
+
+        # Getting path to directory
+        path_dir = os.path.dirname(path)
+
+        # Creating new directory
+        if not os.path.exists(path_dir):
+            if create_directory:
+                os.makedirs(path_dir)
+            else:
+                raise LookupError('Directory not found. Pass create_directory=True to create a new directory')
+
+        if not overwrite_file:
+            if os.path.exists(path):
+                raise FileExistsError(
+                    "The file already exists. Pass overwrite_file=True to overwrite the existing file")
+
     # Creating WMS map object
     wms_map = load_as_map(url=url,
                           layer=layer,
@@ -395,6 +467,7 @@ def load_wfs(url: str) -> owslib.wfs.WebFeatureService:
     Example
     _______
 
+        >>> # Loading Libraries and WFS Service
         >>> import gemgis as gg
         >>> wfs = gg.web.load_wfs(url="https://nibis.lbeg.de/net3/public/ogc.ashx?NodeId=476&Service=WFS&")
         >>> wfs
@@ -449,6 +522,7 @@ def load_as_gpd(url: str,
     Example
     _______
 
+        >>> # Loading Libraries and WFS Service as GeoDataFrame
         >>> import gemgis as gg
         >>> wfs = gg.web.load_as_gpd(url="https://nibis.lbeg.de/net3/public/ogc.ashx?NodeId=476&Service=WFS&")
         >>> wfs
@@ -459,6 +533,7 @@ def load_as_gpd(url: str,
     ________
 
         load_wfs : Load WFS Service
+
     """
 
     # Checking that the url is of type string
@@ -522,6 +597,7 @@ def load_wcs(url: str) -> owslib.wcs.WebCoverageService:
     Example
     _______
 
+        >>> # Loading Libraries and WCS Service
         >>> import gemgis as gg
         >>> wcs = gg.web.load_wms(url='https://www.wcs.nrw.de/geobasis/wcs_nw_dgm')
         >>> wcs
@@ -585,11 +661,13 @@ def create_request(wcs_url: str,
     Example
     _______
 
+        >>> # Loading Libraries and WCS Service
         >>> import gemgis as gg
         >>> wcs = gg.web.load_wms(url='https://www.wcs.nrw.de/geobasis/wcs_nw_dgm')
         >>> wcs
         <owslib.coverage.wcs201.WebCoverageService_2_0_1 at 0x27fc64783d0>
 
+        >>> # Creating Request for WCS Service
         >>> url = gg.web.create_request(url=wcs.url, version=wcs.version, identifier='nw_dgm', form='image/tiff', extent=[0, 1000, 0, 1000], name='test.tif'])
 
     See Also
@@ -635,7 +713,9 @@ def create_request(wcs_url: str,
 
 
 def load_as_file(url: str,
-                 path: str):
+                 path: str,
+                 overwrite_file: bool = False,
+                 create_directory: bool = False):
     """Executing WCS request and downloading file into specified folder
 
     Parameters
@@ -647,16 +727,27 @@ def load_as_file(url: str,
         path: str
             Path where file is saved, e.g. ``path='tile.tif'``
 
+        overwrite_file : bool
+            Variable to overwrite an already existing file.
+            Options include: ``True`` or ``False``, default set to ``False``
+
+        create_directory : bool
+            Variable to create a new directory of directory does not exist
+            Options include: ``True`` or ``False``, default set to ``False``
+
     Example
     _______
 
+        >>> # Loading Libraries and WCS Service
         >>> import gemgis as gg
         >>> wcs = gg.web.load_wms(url='https://www.wcs.nrw.de/geobasis/wcs_nw_dgm')
         >>> wcs
         <owslib.coverage.wcs201.WebCoverageService_2_0_1 at 0x27fc64783d0>
 
+        >>> # Creating Request for WCS Service
         >>> url = gg.web.create_request(url=wcs.url, version=wcs.version, identifier='nw_dgm', form='image/tiff', extent=[0, 1000, 0, 1000], name='test.tif'])
 
+        >>> # Downloading file from WCS Service
         >>> gg.web.load_as_file(url=url, path='tile.tif')
 
     See Also
@@ -676,6 +767,28 @@ def load_as_file(url: str,
     if not isinstance(path, str):
         raise TypeError('Path must be of type string')
 
+    # Getting the absolute path
+    path = os.path.abspath(path=path)
+
+    # Checking that the file has the correct file ending
+    if not path.endswith(".tif"):
+        raise TypeError("The raster must be saved as .tif file")
+
+    # Getting path to directory
+    path_dir = os.path.dirname(path)
+
+    # Creating new directory
+    if not os.path.exists(path_dir):
+        if create_directory:
+            os.makedirs(path_dir)
+        else:
+            raise LookupError('Directory not found. Pass create_directory=True to create a new directory')
+
+    if not overwrite_file:
+        if os.path.exists(path):
+            raise FileExistsError(
+                "The file already exists. Pass overwrite_file=True to overwrite the existing file")
+
     # Executing request and downloading files to the specified folder
     urllib.request.urlretrieve(url, path)
 
@@ -717,11 +830,13 @@ def load_as_files(wcs_url: str,
     Example
     _______
 
+        >>> # Loading Libraries and WCS Service
         >>> import gemgis as gg
         >>> wcs = gg.web.load_wms(url='https://www.wcs.nrw.de/geobasis/wcs_nw_dgm')
         >>> wcs
         <owslib.coverage.wcs201.WebCoverageService_2_0_1 at 0x27fc64783d0>
 
+        >>> # Downloading files from WCS Service
         >>> gg.web.load_as_files(wcs_url=wcs.url, version=wcs.version, form='image/tiff', extent=[0, 10000, 0, 10000], size=2000, path='tile.tif')
 
     See Also
